@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = 0.15;
+our $VERSION = 0.16;
 
 =head1 NAME
 
@@ -193,9 +193,11 @@ Exactly one leading slash will be prepended no matter what you do.
 =over
 
 =item * method - list of allowed HTTP methods.
-Default is [GET, POST, HEAD].
+Default is [GET, POST].
 Multiple handles can be defined for the same path, provided that
 methods do not intersect.
+HEAD method is automatically handled if GET is present, however,
+one MAY define a separate HEAD handler explicitly.
 
 =item * path_info_regex => qr/.../ - allow URI subpaths matching
 the specified expression to be handled by this handler.
@@ -227,6 +229,9 @@ This will be displayed if application called with --list (see L<MVC::Neaf::CLI>)
 
 =back
 
+Also, any number of dash-prefixed keys MAY be present.
+This is totally the same as putting them into C<default> hash.
+
 =cut
 
 my $year = 365 * 24 * 60 * 60;
@@ -248,6 +253,10 @@ sub route {
     my (%args) = @_;
     $self = $Inst unless ref $self;
 
+    # minus-prefixed keys are typically defaults
+    $_ =~ /^-/ and $args{default}{$_} = delete $args{$_}
+        for keys %args;
+
     # kill extra args
     my @extra = grep { !$known_route_args{$_} } keys %args;
     $self->_croak( "Unexpected keys in route setup: @extra" )
@@ -255,7 +264,7 @@ sub route {
 
     $path = canonize_path( $path );
 
-    _listify( \$args{method}, qw( GET POST HEAD ) );
+    _listify( \$args{method}, qw( GET POST ) );
 
     my @dupe = grep { exists $self->{route}{$path}{$_} } @{ $args{method} };
     $self->_croak( "Attempting to set duplicate handler for [@dupe] "
@@ -748,8 +757,15 @@ sub run {
 
     $self->{route_re} ||= $self->_make_route_re;
 
+    # Add implicit HEAD for all GETs via shallow copy
+    foreach my $node (values %{ $self->{route} }) {
+        $node->{GET} or next;
+        $node->{HEAD} ||= { %{ $node->{GET} }, my_method => 'HEAD' };
+    };
+
     # initialize stuff if first run
     # TODO don't allow modification after lock
+    # Please bear in mind that $_[0] in callbacks is ALWAYS the Request object
     if (!$self->{lock}) {
         if (my $engine = $self->{session_handler}) {
             $self->add_hook( pre_route => sub {
@@ -757,8 +773,7 @@ sub run {
             }, prepend => 1 );
             if (my $key = $self->{session_view_as}) {
                 $self->add_hook( pre_render => sub {
-                    my $sess = $_[0]->session(1);
-                    $sess and $_[0]->reply->{$key} = $sess;
+                    $_[0]->reply->{$key} = $_[0]->load_session;
                 }, prepend => 1 );
             };
         };
@@ -789,12 +804,11 @@ sub run {
         };
     } else {
         # void context - CGI called.
-        require MVC::Neaf::Request::CGI;
-
         if (@ARGV) {
             require MVC::Neaf::CLI;
-            MVC::Neaf::CLI->run;
+            MVC::Neaf::CLI->run($self);
         } else {;
+            require MVC::Neaf::Request::CGI;
             my $req = MVC::Neaf::Request::CGI->new;
             $self->handle_request( $req );
         };
@@ -1548,9 +1562,9 @@ sub _log_error {
     warn $msg;
 };
 
-=head2 run_test( \%PSGI_ENV )
+=head2 run_test( \%PSGI_ENV, %options )
 
-=head2 run_test( "/path?param=value" )
+=head2 run_test( "/path?param=value", %options )
 
 Run a PSGI request and return a list of
 C<($status, HTTP::Headers, $whole_content )>.
@@ -1559,10 +1573,18 @@ Just as the name suggests, useful for testing only (it reduces boilerplate).
 
 Continuation responses are supported.
 
+%options may include:
+
+=over
+
+=item * override = \%hash - force certain data in ENV
+
+=back
+
 =cut
 
 sub run_test {
-    my ($self, $env) = @_;
+    my ($self, $env, %opt) = @_;
 
     if (!ref $env) {
         $env =~ /^(.*?)(?:\?(.*))?$/;
@@ -1576,6 +1598,9 @@ sub run_test {
             PATH_INFO => $1,
         }
     };
+    # TODO more civilized stuff like cookies, headers...
+    $env->{$_} = $opt{override}{$_} for keys %{ $opt{override} };
+
     my $ret = $self->run->( $env );
     if (ref $ret eq 'CODE') {
         # PSGI functional interface used.
