@@ -2,7 +2,7 @@ package MVC::Neaf::X::Session::Base;
 
 use strict;
 use warnings;
-our $VERSION = '0.2800_01';
+our $VERSION = 0.16;
 
 =head1 DESCRIPTION
 
@@ -23,10 +23,10 @@ Not Even A Framework.
 
 =cut
 
+use parent qw(MVC::Neaf::X::Session);
+use JSON;
 use Carp;
 
-use MVC::Neaf::Util qw(encode_json decode_json);
-use parent qw(MVC::Neaf::X::Session);
 
 =head2 new( %options )
 
@@ -221,18 +221,4 @@ sub store {
     $self->my_croak("unimplemented");
 };
 
-=head1 LICENSE AND COPYRIGHT
-
-This module is part of L<MVC::Neaf> suite.
-
-Copyright 2016-2023 Konstantin S. Uvarin C<khedin@cpan.org>.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See L<http://dev.perl.org/licenses/> for more information.
-
-=cut
-
 1;
