@@ -2,7 +2,7 @@ package MVC::Neaf::X::Session::Cookie;
 
 use strict;
 use warnings;
-our $VERSION = '0.2800_01';
+our $VERSION = 0.16;
 
 =head1 NAME
 
@@ -12,18 +12,12 @@ MVC::Neaf::X::Session::Cookie - Stateless cookie-based session for Neaf
 
 Use this module as a session handler in a Neaf app.
 
-The session data is stored within user's cookies without encryption.
-However, it is signed with a key only known to the application owner.
-So the session can be read, but not tampered with.
-
-Please take these concern into account, or better use server-side storage.
-
 =head1 METHODS
 
 =cut
 
-use MVC::Neaf::Util qw( encode_b64 decode_b64 );
-use Digest::SHA qw( hmac_sha224 );
+use MIME::Base64 qw( encode_base64 decode_base64 );
+use Digest::SHA;
 
 use parent qw( MVC::Neaf::X::Session::Base );
 
@@ -33,7 +27,8 @@ use parent qw( MVC::Neaf::X::Session::Base );
 
 =over
 
-=item * key (required) - a secret text string used to sign session data.
+=item * key (required) - a security key to prevent tampering with
+the cookie data.
 This should be the same throughout the application.
 
 =item * hmac_function - HMAC to be used, default is hmac_sha224_base64
@@ -47,7 +42,7 @@ sub new {
 
     $opt{key} or $class->my_croak( "key option is required" );
 
-    $opt{hmac_function} ||= sub { encode_b64( hmac_sha224( @_ ) ) };
+    $opt{hmac_function} ||= \&Digest::SHA::hmac_sha224_base64;
 
     return $class->SUPER::new( %opt );
 };
@@ -61,8 +56,8 @@ Create a cookie from $data hash. Given $id is ignored.
 sub store {
     my ($self, $id, $data) = @_;
 
-    # TODO 0.90 Make universal HMAC mechanism for ALL cookies
-    my $str = encode_b64($data);
+    # TODO Make universal HMAC for ALL cookies
+    my $str = encode_base64($data);
     $str =~ s/\s//gs;
     $str .= "~".$self->get_expire;
     my $sum = $self->{hmac_function}->( $str, $self->{key} );
@@ -84,7 +79,7 @@ sub fetch {
     return unless $key;
     return unless $self->{hmac_function}->( "$str~$time", $self->{key} ) eq $key;
 
-    return { strfy => decode_b64($str), expire => $time };
+    return { strfy => decode_base64($str), expire => $time };
 };
 
 =head2 get_session_id
@@ -95,18 +90,4 @@ Replaced by a stub - we'll generate ID from data anyway.
 
 sub get_session_id { return 'Cookie Session Need No Id' };
 
-=head1 LICENSE AND COPYRIGHT
-
-This module is part of L<MVC::Neaf> suite.
-
-Copyright 2016-2023 Konstantin S. Uvarin C<khedin@cpan.org>.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See L<http://dev.perl.org/licenses/> for more information.
-
-=cut
-
 1;
